@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         Minolta.pm
 #
-# Description:  Definitions for Minolta EXIF Maker Notes
+# Description:  Minolta EXIF maker notes tags
 #
 # Revisions:    04/06/2004 - P. Harvey Created
 #               09/09/2005 - P. Harvey Added ability to write MRW files
@@ -22,7 +22,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.19';
+$VERSION = '1.21';
 
 %Image::ExifTool::Minolta::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -354,11 +354,15 @@ $VERSION = '1.19';
     },
     21 => {
         Name => 'MinoltaDate',
+        Groups => { 2 => 'Time' },
+        Shift => 'Time',
         ValueConv => 'sprintf("%4d:%.2d:%.2d",$val>>16,($val&0xff00)>>8,$val&0xff)',
         ValueConvInv => 'my @a=($val=~/(\d+):(\d+):(\d+)/); @a ? ($a[0]<<16)+($a[1]<<8)+$a[2] : undef',
     },
     22 => {
         Name => 'MinoltaTime',
+        Groups => { 2 => 'Time' },
+        Shift => 'Time',
         ValueConv => 'sprintf("%.2d:%.2d:%.2d",$val>>16,($val&0xff00)>>8,$val&0xff)',
         ValueConvInv => 'my @a=($val=~/(\d+):(\d+):(\d+)/); @a ? ($a[0]<<16)+($a[1]<<8)+$a[2] : undef',
     },
@@ -609,20 +613,34 @@ sub ProcessMRW($$)
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
-    my $data;
+    my $verbose = $exifTool->Options('Verbose');
+    my $out = $exifTool->Options('TextOut');
+    my ($data, $err);
 
     $raf->Read($data,8) == 8 or return 0;
     $data =~ /^\0MRM/ or return 0;
+    $exifTool->SetFileType();
     SetByteOrder('MM');
     $outfile and $exifTool->InitWriteDirs('TIFF'); # use same write dirs as TIFF
     my $offset = Get32u(\$data, 4) + 8;
     my $pos = 8;
+    my $rtnVal = 1;
+    $verbose and printf $out "  [Data Offset: 0x%x]\n", $offset;
     # decode MRW structure to locate start of TIFF-format image (ref 5)
     while ($pos < $offset) {
-        $raf->Read($data,8) == 8 or return 0;
+        $raf->Read($data,8) == 8 or $err = 1, last;
         $pos += 8;
         my $tag = substr($data, 0, 4);
         my $len = Get32u(\$data, 4);
+        if ($verbose) {
+            print $out "MRW ",$exifTool->Printable($tag)," segment ($len bytes):\n";
+            if ($verbose > 2) {
+                $raf->Read($data,$len) == $len and $raf->Seek($pos,0) or $err = 1, last;
+                my %parms = (Addr => $pos, Out => $out);
+                $parms{MaxLen} = 96 unless $verbose > 3;
+                Image::ExifTool::HexDump(\$data,undef,%parms);
+            }
+        }
         if ($tag eq "\0TTW") {
             # parse the TIFF structure after the TTW tag
             my %dirInfo = (
@@ -633,9 +651,12 @@ sub ProcessMRW($$)
             # rewrite the EXIF information (plus the file header)
             my $buff = '';
             $dirInfo{OutFile} = \$buff if $outfile;
-            my $rtnVal = $exifTool->ProcessTIFF(\%dirInfo);
-            return $rtnVal unless $outfile;
-            if ($rtnVal == 1) {
+            my $result = $exifTool->ProcessTIFF(\%dirInfo);
+            if ($result < 0) {
+                $rtnVal = -1;
+            } elsif (not $result) {
+                $err = 1;
+            } elsif ($outfile) {
                 # adjust offset for new EXIF length
                 my $newLen = length($buff) - $pos;
                 $offset += $newLen - $len;
@@ -644,17 +665,19 @@ sub ProcessMRW($$)
                 Write($outfile, $buff) or $rtnVal = -1;
                 # rewrite the rest of the file
                 $pos += $len;
-                $raf->Seek($pos, 0) or return 0;
+                $raf->Seek($pos, 0) or $err = 1, last;
                 while ($raf->Read($buff, 65536)) {
                     Write($outfile, $buff) or $rtnVal = -1;
                 }
+                last;   # all done
             }
-            return $rtnVal;
+            last unless $verbose;
         }
         $pos += $len;
-        $raf->Seek($pos, 0) or return 0;
+        $raf->Seek($pos, 0) or $err = 1, last;
     }
-    return 0;
+    $err and $exifTool->Error("MRW format error");
+    return $rtnVal;
 }
 
 1;  # end
@@ -663,7 +686,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::Minolta - Definitions for Minolta EXIF maker notes
+Image::ExifTool::Minolta - Minolta EXIF maker notes tags
 
 =head1 SYNOPSIS
 
@@ -673,11 +696,11 @@ This module is loaded automatically by Image::ExifTool when required.
 
 This module contains definitions required by Image::ExifTool to interpret
 Minolta and Konica-Minolta maker notes in EXIF information, and to read
-and write Minolta RAW (MRW) files.
+and write Minolta RAW (MRW) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
